/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright 2017 NXP
 */

#ifndef _ASM_ARCH_IMX8_POWER_DOMAIN_H
#define _ASM_ARCH_IMX8_POWER_DOMAIN_H

#include <asm/arch/sci/types.h>

struct imx8_power_domain_plat {
	sc_rsrc_t resource_id;
};

#endif
