/* SPDX-License-Identifier: GPL-2.0 */
/*
 * (C) Copyright 2013-2016, NVIDIA CORPORATION.
 */

#ifndef _TEGRA186_TEGRA_H_
#define _TEGRA186_TEGRA_H_

#define GICD_BASE		0x03881000	/* Generic Int Cntrlr Distrib */
#define GICC_BASE		0x03882000	/* Generic Int Cntrlr CPU I/F */
#define NV_PA_SDRAM_BASE	0x80000000

#include <asm/arch-tegra/tegra.h>

#endif
