/*
 * Copyright (C) 2016 Icenowy Zheng <icenowy@aosc.io>
 *
 * SPDX-License-Identifier: (GPL-2.0+ OR MIT)
 */

#ifndef _DT_BINDINGS_CLOCK_SUN8I_DE2_H_
#define _DT_BINDINGS_CLOCK_SUN8I_DE2_H_

#define CLK_BUS_MIXER0		0
#define CLK_BUS_MIXER1		1
#define CLK_BUS_WB		2

#define CLK_MIXER0		6
#define CLK_MIXER1		7
#define CLK_WB			8

#define CLK_BUS_ROT		9
#define CLK_ROT			10

#endif /* _DT_BINDINGS_CLOCK_SUN8I_DE2_H_ */
