/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2017, STMicroelectronics - All Rights Reserved
 * Author(s): Patrice Chotard, <patrice.chotard@foss.st.com> for STMicroelectronics.
 */

#ifndef _ASM_ARCH_HARDWARE_H
#define _ASM_ARCH_HARDWARE_H

/*
 * This empty files is needed to not break compilation
 * Some common drivers to STM32F4/F7 and H7 include a stm32.h file
 * Some cleanup need to be done to communalize all the following
 * stm32.h files:
 *
 * arch/arm/include/asm/arch-stm32f1/stm32.h
 * arch/arm/include/asm/arch-stm32f4/stm32.h
 * arch/arm/include/asm/arch-stm32f7/stm32.h
 */
#endif /* _ASM_ARCH_HARDWARE_H */
