/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * MIPS Relocations
 *
 * Copyright (c) 2017 Imagination Technologies Ltd.
 */

#ifndef __ASM_MIPS_RELOCS_H__
#define __ASM_MIPS_RELOCS_H__

#define R_MIPS_NONE		0xbeef7531
#define R_MIPS_32		2
#define R_MIPS_26		4
#define R_MIPS_HI16		5
#define R_MIPS_LO16		6
#define R_MIPS_PC16		10
#define R_MIPS_64		18
#define R_MIPS_HIGHER		28
#define R_MIPS_HIGHEST		29
#define R_MIPS_PC21_S2		60
#define R_MIPS_PC26_S2		61

#endif /* __ASM_MIPS_RELOCS_H__ */
