/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2018 MediaTek Inc.
 */

#ifndef _DT_BINDINGS_MT7623_POWER_H
#define _DT_BINDINGS_MT7623_POWER_H

#define MT7623_POWER_DOMAIN_CONN	0
#define MT7623_POWER_DOMAIN_DISP	1
#define MT7623_POWER_DOMAIN_MFG		2
#define MT7623_POWER_DOMAIN_VDEC	3
#define MT7623_POWER_DOMAIN_ISP		4
#define MT7623_POWER_DOMAIN_BDP		5
#define MT7623_POWER_DOMAIN_ETH		6
#define MT7623_POWER_DOMAIN_HIF		7
#define MT7623_POWER_DOMAIN_IFR_MSC	8

#endif /* _DT_BINDINGS_MT7623_POWER_H */
