/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2016 Cadence Design Systems Inc.
 */

#ifndef _MIPS_ATOMIC_H
#define _MIPS_ATOMIC_H

#include <asm/system.h>
#include <asm-generic/atomic.h>

#endif
