/* SPDX-License-Identifier: Intel */
/*
 * Copyright 2019 Google LLC
 */

#ifndef __FSP_CONFIGS_H__
#define __FSP_CONFIGS_H__

#define FSPT_UPD_SIGNATURE	0x545F4450554C5041	/* 'APLUPD_T' */
#define FSPM_UPD_SIGNATURE	0x4D5F4450554C5041	/* 'APLUPD_M' */
#define FSPS_UPD_SIGNATURE	0x535F4450554C5041	/* 'APLUPD_S' */
#define VBT_SIGNATURE		0x54425624

#endif
