/* DO NOT EDIT THIS FILE
 * Automatically generated by toolchain/trunk/proc-defs/sh/create-arch-headers.sh
 * DO NOT EDIT THIS FILE
 */

#ifndef __MACH_GLOB_BLACKFIN__
#define __MACH_GLOB_BLACKFIN__

#include "blackfin_def.h"
#ifndef __ASSEMBLY__
#include "blackfin_cdef.h"
#endif
#include "blackfin_local.h"

#endif /* __MACH_GLOB_BLACKFIN__ */
