/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2018 MediaTek Inc.
 */

#ifndef __MEDIATEK_MISC_H_
#define __MEDIATEK_MISC_H_

#define VER_BASE		0x08000000
#define VER_SIZE		0x10

#define APHW_CODE		0x00
#define APHW_SUBCODE		0x04
#define APHW_VER		0x08
#define APSW_VER		0x0c

#endif /* __MEDIATEK_MISC_H_ */
