/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright 2015-2016 Freescale Semiconductor, Inc.
 * Copyright 2017 NXP
 */

/** @file
 *  Contains all the defines to handle parsing and loading of PE firmware files.
 */
#ifndef __PFE_FIRMWARE_H__
#define __PFE_FIRMWARE_H__

int pfe_firmware_init(void);
void pfe_firmware_exit(void);

#endif
