/*
 * Port D Masks
 */

#ifndef __BFIN_PERIPHERAL_PORT_D__
#define __BFIN_PERIPHERAL_PORT_D__

#define PD0		(1 << 0)
#define PD1		(1 << 1)
#define PD2		(1 << 2)
#define PD3		(1 << 3)
#define PD4		(1 << 4)
#define PD5		(1 << 5)
#define PD6		(1 << 6)
#define PD7		(1 << 7)
#define PD8		(1 << 8)
#define PD9		(1 << 9)
#define PD10		(1 << 10)
#define PD11		(1 << 11)
#define PD12		(1 << 12)
#define PD13		(1 << 13)
#define PD14		(1 << 14)
#define PD15		(1 << 15)

#endif
