/*
** File generated by bin2inc from "test"
** DO NOT EDIT
*/

#ifndef _code_
unsigned char code[] = {
	0xb8,0x02,0x4f,0xbb,0x12,0xc1,0xcd,0x10,0xb8,0x00,0x40,0x8e,0xc0,0xbf,0x00,0x00,
	0xb8,0x01,0x4f,0xb9,0x12,0xc1,0xcd,0x10,0xf4,
};

#define code_COUNT 25

#endif
