/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * This header provides constants for Qualcomm Snapdragon pinctrl bindings.
 *
 * (C) Copyright 2018 Ramon Fried <ramon.fried@gmail.com>
 *
 */

#ifndef _DT_BINDINGS_PINCTRL_SNAPDRAGON_H
#define _DT_BINDINGS_PINCTRL_SNAPDRAGON_H

/* GPIO Drive Strength */
#define DRIVE_STRENGTH_2MA        0
#define DRIVE_STRENGTH_4MA        1
#define DRIVE_STRENGTH_6MA        2
#define DRIVE_STRENGTH_8MA        3
#define DRIVE_STRENGTH_10MA       4
#define DRIVE_STRENGTH_12MA       5
#define DRIVE_STRENGTH_14MA       6
#define DRIVE_STRENGTH_16MA       7

#endif
