/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2016, STMicroelectronics - All Rights Reserved
 * Author(s): Vikas Manocha, <vikas.manocha@st.com> for STMicroelectronics.
 */

#ifndef _STM32_GPIO_H_
#define _STM32_GPIO_H_

#include <asm/arch-stm32/gpio.h>

#endif /* _STM32_GPIO_H_ */
