/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2020 SiFive, Inc.
 *
 * Authors:
 *   Pragnesh Patel <pragnesh.patel@sifve.com>
 */

#ifndef _SPL_SIFIVE_H
#define _SPL_SIFIVE_H

int spl_soc_init(void);

#endif /* _SPL_SIFIVE_H */
