/* SPDX-License-Identifier: GPL-2.0+ */
/* include/linux/usb/dwc3-omap.h
 *
 * Copyright (c) 2014 Texas Instruments Incorporated - http://www.ti.com
 *
 * Designware SuperSpeed Glue
 */

#ifndef __DWC3_OMAP_H_
#define __DWC3_OMAP_H_

enum dwc3_omap_utmi_mode {
	DWC3_OMAP_UTMI_MODE_UNKNOWN = 0,
	DWC3_OMAP_UTMI_MODE_HW,
	DWC3_OMAP_UTMI_MODE_SW,
};

#endif /* __DWC3_OMAP_H_ */
