/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2000
 * Murray Jensen <Murray.Jensen@csiro.au>
 */

#include <stdarg.h>

extern char *pname;

extern void Warning(char *, ...);
extern void Error(char *, ...);
extern void Perror(char *, ...);
