/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright 2014 Broadcom Corporation.
 */

#ifndef __IPROC_COMMON_CONFIGS_H
#define __IPROC_COMMON_CONFIGS_H

#include <linux/stringify.h>

/* Architecture, CPU, chip, etc */
#define CONFIG_IPROC

/* Memory Info */
#define CONFIG_SYS_SDRAM_BASE		0x61000000

#endif /* __IPROC_COMMON_CONFIGS_H */
