/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (c) 2011 The Chromium OS Authors.
 */

/* Tegra20 timer functions */

#ifndef _TEGRA_TIMER_H
#define _TEGRA_TIMER_H

/* returns the current monotonic timer value in microseconds */
unsigned long timer_get_us(void);

#endif
