/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2017 Andes Technology Corporation
 * Rick Chen, Andes Technology Corporation <rick@andestech.com>
 */

#ifndef __ASM_RISCV_SYSTEM_H
#define __ASM_RISCV_SYSTEM_H

/*
 * Interrupt configuring macros.
 *
 * TODO
 *
 */

#endif	/* __ASM_RISCV_SYSTEM_H */
