/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2017 Álvaro Fernández Rojas <noltari@gmail.com>
 *
 * Derived from Broadcom GPL Source Code:
 *	Copyright (C) Broadcom Corporation
 */

#ifndef __DT_BINDINGS_RESET_BCM3380_H
#define __DT_BINDINGS_RESET_BCM3380_H

#define BCM3380_RST0_SPI	0
#define BCM3380_RST0_PCM	13

#endif /* __DT_BINDINGS_RESET_BCM3380_H */
