/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2018 Álvaro Fernández Rojas <noltari@gmail.com>
 *
 * Derived from linux/arch/mips/include/asm/mach-bcm63xx/bcm63xx_regs.h
 */

#ifndef __DT_BINDINGS_RESET_BCM6362_H
#define __DT_BINDINGS_RESET_BCM6362_H

#define BCM6362_RST_SPI		0
#define BCM6362_RST_IPSEC	1
#define BCM6362_RST_EPHY	2
#define BCM6362_RST_SAR		3
#define BCM6362_RST_ENETSW	4
#define BCM6362_RST_USBD	5
#define BCM6362_RST_USBH	6
#define BCM6362_RST_PCM		7
#define BCM6362_RST_PCIE_CORE	8
#define BCM6362_RST_PCIE	9
#define BCM6362_RST_PCIE_EXT	10
#define BCM6362_RST_WLAN_SHIM	11
#define BCM6362_RST_DDR_PHY	12
#define BCM6362_RST_FAP		13
#define BCM6362_RST_WLAN_UBUS	14

#endif /* __DT_BINDINGS_RESET_BCM6362_H */
