/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2017 Álvaro Fernández Rojas <noltari@gmail.com>
 */

#include <configs/bmips_common.h>
#include <configs/bmips_bcm6358.h>

#define CONFIG_REMAKE_ELF

